#include "hbqtgui.ch"
#include "hmg.ch"

#define HMG_DIALOGBOX_GETFILE   1
#define HMG_DIALOGBOX_PUTFILE   2
#define HMG_DIALOGBOX_GETFOLDER 3

/*
 *  L_GetFolder()
 *  Backward compatibility ok.
 *  Extensions. optional
 *  [cInitFolder ]. Open the dialog box opening [cInitFolder] path.
*/
FUNCTION L_GetFolder( cTitle, cInitFolder )

   LOCAL aRet := L_GetSetFile( HMG_DIALOGBOX_GETFOLDER, cTitle, cInitFolder )
   LOCAL cFolderName := IF( EMPTY( aRet ), "", aRet[1] )

   RETURN cFolderName

/*
  * L_GetFile()
  * Backward compatibility is ok, except with lNoChangeDir !!
  * lNoChangeDir. The equivalent in qt is ..?
  * Extensions. optional
  * ---REMOVED--->  [ cSuffix ]. Predefined extension( suffix ) to use with selected file. REMOVED because this QT option can be used ONLY when saving
  *   [ cFileName ]. Predefined filename to show in dialog box.
*/
FUNCTION L_GetFile( xFilter, cTitle, cInitFolder, lMultiSelect, lNoChangeDir, cFileName )

   LOCAL aRet := L_GetSetFile( HMG_DIALOGBOX_GETFILE, cTitle, cInitFolder, xFilter, cFileName, lMultiSelect )
   LOCAL xReturn

   HB_SYMBOL_UNUSED( lNoChangeDir )

   IF ! hb_IsLogical( lMultiSelect ) ; lMultiSelect := .F. ; ENDIF

   IF lMultiSelect
      xReturn := ACLONE( aRet )
   ELSE
      xReturn := IF( EMPTY( aRet ), "", aRet[1] )
   ENDIF

   RETURN xReturn

/*
 *  PutFile()
 *  Backward compatibility ok.
 *  Extensions. optional
 *    [ cSuffix ]. Predefined extension( suffix ) to use with selected file.
 * ---REMOVED--->  [ cFileName ]. Predefined filename to show in dialog box. REMOVED because this QT option can be used ONLY when getting
*/
FUNCTION L_PutFile( xFilter, cTitle, cInitFolder, lNoChangeDir, cSuffix )

   LOCAL aRet := L_GetSetFile( HMG_DIALOGBOX_PUTFILE, cTitle, cInitFolder, xFilter, cSuffix )
   LOCAL cFileReturned := IF( EMPTY( aRet ), "", aRet[1] )

   HB_SYMBOL_UNUSED( lNoChangeDir )

   RETURN cFileReturned

/*
 * nMode
 * cTitle: window Title
 * cInitFolder: starting path
 * xFilter: can be like HMG3 style eg. { {'Jpeg Files','*.jpg;*.png'}, {'BitMap Files','*.Bmp'}, {'Text Files','*.Txt'} }
 *          or can be QT style eg. "Jpeg Files (*.jpg *.png);;BitMap Files (*.Bmp);;Text Files (*.txt)"
 * cArg1: when nMode==HMG_DIALOGBOX_PUTFILE default file extension
 *        when nMode==HMG_DIALOGBOX_GETFILE predefined filename to be selected
 * lMutliSelect: when nMode==HMG_DIALOGBOX_GETFILE .T. can be selected more than one file else (default) .F.
*/
FUNCTION L_GetSetFile( nMode, cTitle, cInitFolder, xFilter, cArg1, lMultiSelect )

   LOCAL oFD, qList, cFilter := "( * )"
   LOCAL aRet := {}, nX

   IF ! hb_IsNumeric( nMode )        ; nMode        := HMG_DIALOGBOX_GETFILE ; ENDIF
   IF ! hb_IsChar( cTitle )          ; cTitle       := "Select"              ; ENDIF
   IF ! hb_IsChar( cInitFolder )     ; cInitFolder  := GetUserHomeFolder()   ; ENDIF
   IF ! hb_IsLogical( lMultiSelect ) ; lMultiSelect := .F.                   ; ENDIF

   // to ensure nMode is acceptable
   nMode := IF( nMode >= HMG_DIALOGBOX_GETFILE .AND. nMode <= HMG_DIALOGBOX_GETFOLDER, nMode, HMG_DIALOGBOX_GETFILE )

   // to ensure cInitFolder is acceptable
   cInitFolder := IF( QDir():exists( cInitFolder ), cInitFolder, GetUserHomeFolder() )

   // TOFIX: at this moment only GETFILE can accept more than one file
   lMultiSelect := IF( nMode == HMG_DIALOGBOX_GETFILE, lMultiSelect, .F. )

   oFD := QFileDialog()
   oFD:setWindowTitle( cTitle )
   oFD:setDirectory( cInitFolder )

   DO CASE

      CASE nMode == HMG_DIALOGBOX_GETFOLDER   // Don't use the native OS Dialog !
         oFD:setAcceptMode( QFileDialog_AcceptOpen )
         oFD:setFileMode( QFileDialog_Directory )
         oFD:setOptions( QFileDialog_ShowDirsOnly )

      CASE nMode == HMG_DIALOGBOX_GETFILE
         oFD:setAcceptMode( QFileDialog_AcceptOpen )
         IF lMultiSelect == .T.
            oFD:setFileMode( QFileDialog_ExistingFiles )
         ELSE
            oFD:setFileMode( QFileDialog_ExistingFile )
         ENDIF

      CASE nMode == HMG_DIALOGBOX_PUTFILE
         // _AcceptSave it's the right way for two reason
         //   1. Caption button is "Save"
         //   2. If you select an existing file QT automation show a dialog box with two buttons (Yes, No) and "Do you want to replace...?"
         //   But, at this moment, we can't modify anything about the automated box dialog; language, for example.
         oFD:setAcceptMode( QFileDialog_AcceptSave )
/*         oFD:setAcceptMode( QFileDialog_AcceptOpen ) */
         oFD:setFileMode( QFileDialog_AnyFile )

   ENDCASE

   IF nMode == HMG_DIALOGBOX_GETFILE .OR. nMode == HMG_DIALOGBOX_PUTFILE
      // Filters
      //  QT style
      IF hb_IsChar( xFilter ) ; cFilter := xFilter ; ENDIF
      // HMG style
      IF hb_IsArray( xFilter )
         cFilter := ""
         FOR nX := 1 TO LEN( xFilter )
            cFilter += IF( nX <> 1, ";;", "" ) + xFilter[ nX ][1] + " (" + STRTRAN( xFilter[ nX ][2], ";", " " ) + ")"
         NEXT
      ENDIF
      oFD:setNameFilter( cFilter )

      // FileName predefined when selecting
      IF nMode == HMG_DIALOGBOX_GETFILE .AND. hb_IsChar( cArg1 ) == .T.
         oFD:SelectFile( cArg1 )
      ENDIF
      // Default extension when saving
      IF nMode == HMG_DIALOGBOX_PUTFILE .AND. hb_IsChar( cArg1 ) == .T.
         oFD:setDefaultSuffix( cArg1 )
      ENDIF
   ENDIF

   IF oFD:exec() == QDialog_Accepted
      qList := oFD:selectedFiles()
      IF QList:isEmpty() == .F.
         FOR nX := 1 TO QList:size()
            AADD( aRet, QDir():toNativeSeparators( QList:at( nX - 1 ) ) )
         NEXT
      ENDIF
   ENDIF

   oFD := NIL
   hb_gcall( .T. )

   RETURN aRet
